#!/bin/sh

cpu_conf=/etc/pandora/conf/dsp.conf
minmhz="$(grep '^min:' $cpu_conf | awk -F\: '{print $2}')"
maxmhz="$(grep '^max:' $cpu_conf | awk -F\: '{print $2}')"
test -z "$minmhz" && minmhz=90
test -z "$maxmhz" && maxmhz=1000

device=/proc/pandora/dsp_mhz_max
curmhz="$(cat $device)"
newmhz="$(cat $device)"

if [ "$1" = "-n" ]; then
    shift
    warn=no
fi

if [ ! -e $device ]; then
    if [ -z "$1" -a -n "$DISPLAY" ]; then
        zenity --info --title="DSP overclocking not supported" --text \
            "Sorry, your kernel does not support DSP overclocking."
    fi
    exit 1
fi

if [ -z "$1" ]; then
    if [ -n "$DISPLAY" ]; then
        newmhz=$(zenity --scale --title "DSP clockspeed" --text "Set DSP clockspeed" --min-value=$minmhz --max-value=$maxmhz --value=$curmhz --step 1)
    else
        echo "Clock speed not specified"
        exit 1
    fi
else
    newmhz=$1
fi

if ! [ 0 -lt "$newmhz" -a "$newmhz" -lt 10000 ]; then
    echo "invalid argument: $newmhz"
    exit 1
fi

if [ "$newmhz" -lt "$minmhz" ]; then
    echo "limiting clock to configured minimum of $minmhz"
    newmhz=$minmhz
fi
if [ "$newmhz" -gt "$maxmhz" ]; then
    echo "limiting clock to configured maximum of $maxmhz"
    newmhz=$maxmhz
fi

# not much else to do here..
echo "$newmhz" > $device
